package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.TotemOfReviving;
import dev.micle.totemofreviving.util.MismatchedVersionException;
import net.minecraft.network.PacketBuffer;
import net.minecraftforge.fml.network.NetworkRegistry;
import net.minecraftforge.fml.network.simple.SimpleChannel;

import java.util.Objects;
import java.util.regex.Pattern;

public class Network {
	public static final String VERSION = TotemOfReviving.MOD_ID + "-net-1";
	private static final Pattern NET_VERSION_PATTERN = Pattern.compile(TotemOfReviving.MOD_ID + "-net-\\d+$");
	private static final Pattern MOD_VERSION_PATTERN = Pattern.compile("Forge-\\d+\\.\\d+\\.\\d+-\\d+\\.\\d+\\.\\d+$");
	
	public static SimpleChannel channel;
	
	public static void init() {
		channel = NetworkRegistry.ChannelBuilder.named(TotemOfReviving.createResourceLocation("network"))
				.clientAcceptedVersions(s -> Objects.equals(s, VERSION))
				.serverAcceptedVersions(s -> Objects.equals(s, VERSION))
				.networkProtocolVersion(() -> VERSION)
				.simpleChannel();
		
		int id = 0;
		channel.messageBuilder(ChangeTargetPacket.class, id++)
				.encoder(ChangeTargetPacket::encode)
				.decoder(ChangeTargetPacket::decode)
				.consumer(ChangeTargetPacket::handle)
				.add();
		channel.messageBuilder(ChargeTotemPacket.class, id++)
				.encoder(ChargeTotemPacket::encode)
				.decoder(ChargeTotemPacket::decode)
				.consumer(ChargeTotemPacket::handle)
				.add();
		channel.messageBuilder(ReviveTargetPacket.class, id++)
				.encoder(ReviveTargetPacket::encode)
				.decoder(ReviveTargetPacket::decode)
				.consumer(ReviveTargetPacket::handle)
				.add();
	}
	
	public static void writeVersionInfo(PacketBuffer buffer, boolean senderIsServer) {
		buffer.writeBoolean(senderIsServer);
		buffer.writeUtf(VERSION);
		buffer.writeUtf(TotemOfReviving.getVersion());
	}
	
	public static void checkVersion(PacketBuffer buffer) {
		boolean senderIsServer = buffer.readBoolean();
		String serverNetVersion = (senderIsServer) ? readNetVersion(buffer) : VERSION;
		String serverModVersion = (senderIsServer) ? readModVersion(buffer) : TotemOfReviving.getVersion();
		String clientNetVersion = (senderIsServer) ? VERSION : readNetVersion(buffer);
		String clientModVersion = (senderIsServer) ? TotemOfReviving.getVersion() : readModVersion(buffer);
		
		if (!serverNetVersion.equals(clientNetVersion) ||
				!serverModVersion.equals(clientModVersion) && !serverModVersion.equals("NONE") && !clientModVersion.equals("NONE")) {
			throw new MismatchedVersionException(
					String.format("The server and client are running different versions of [Micle's Totem of Reviving]. " +
							"Try updating this mod on either the client and or the server. " +
							"Client version is %s (%s). Server version is %s (%s)",
							clientModVersion, clientNetVersion,
							serverModVersion, serverNetVersion)
			);
		}
	}
	
	public static String readNetVersion(PacketBuffer buffer) {
		String netVersion = buffer.readUtf();
		if (!NET_VERSION_PATTERN.matcher(netVersion).matches()) {
			return "UNKNOWN (" + netVersion + ")";
		}
		return netVersion;
	}
	
	public static String readModVersion(PacketBuffer buffer) {
		String modVersion = buffer.readUtf();
		if (!modVersion.equals("NONE") && !MOD_VERSION_PATTERN.matcher(modVersion).matches()) {
			return "UNKNOWN (" + modVersion + ")";
		}
		return modVersion;
	}
}
